// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

mod pty;
pub use self::pty::Pty;

mod terminal;
pub use self::terminal::Terminal;

mod regex;
pub use self::regex::Regex;

mod enums;
#[cfg(feature = "v0_76")]
#[cfg_attr(docsrs, doc(cfg(feature = "v0_76")))]
pub use self::enums::Align;
pub use self::enums::CursorBlinkMode;
pub use self::enums::CursorShape;
pub use self::enums::EraseBinding;
pub use self::enums::Format;
pub use self::enums::PtyError;
pub use self::enums::RegexError;
pub use self::enums::TextBlinkMode;
pub use self::enums::WriteFlags;

mod flags;
pub use self::flags::FeatureFlags;
pub use self::flags::PtyFlags;

pub(crate) mod traits {
    pub use super::terminal::TerminalExt;
}
pub(crate) mod builders {
    pub use super::pty::PtyBuilder;
    pub use super::terminal::TerminalBuilder;
}
